/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

/**@name LIBRARY: unManager.ctl

@author: Jean-Charles Tournier (BE-ICS)

Creation Date: 04-August-2021

Modification History:
  04-08-2021: Jean-Charles Tournier
  - Initial version
**/


/**
 * Gives the manager number used for the 
 * manager unicos_scripts.lst. This manager
 * number needs to match the manager number
 * given as argument in the project console.
 **/
int unManager_getUnicosScriptsManagerNumber(){
  return 100;
}

/**
 * Gives the manager number used for the 
 * manager unDistributedControl.ctl. This manager
 * number needs to match the manager number
 * given as argument in the project console.
 **/
int unManager_getUnDistributedControlManagerNumber(){
  return 101;
}

/**
 * Gives the manager number used for the 
 * manager unBackup.ctl. This manager
 * number needs to match the manager number
 * given as argument in the project console.
 **/
int unManager_getUnBackupManagerNumber(){
  return 102;
}

/**
 * Gives the manager number used for the 
 * manager unDdeviceListUpdate.ctl. This manager
 * number needs to match the manager number
 * given as argument in the project console.
 **/
int unManager_getUnDeviceListUpdateManagerNumber(){
  return 103;
}

/**
 * Given a list of running manager numbers,
 * check if the number associated to unicos_scripts.lst
 * is present in this list.
 * The manager list is normally provided by the WinCC OA internal DPE
 * and this function is used as a callback on a dpConnnect on _Connections.Ctrl.ManNums
 * 
 * @param const dyn_int &diListCtrlManagerNumber: list of manager number
 *                                                the list should be the content of _Connections.Ctrl.ManNums
 * @return true if unicos_scripts.lst manager number is in the list, false otherwise.
 **/
bool unManager_isUnicosScriptsInListOfRunningCtrl( const dyn_int &diListCtrlManagerNumber){
  return dynContains( diListCtrlManagerNumber, unManager_getUnicosScriptsManagerNumber()) > 0;
}

/**
 * Check if the manager number associated to unicos_scripts.lst is 
 * in the list of running manager of the system given as parameter. 
 * 
 * @param string sSystemName: Optional name of the system to be checked.
 *                            If no name is given, the local system name is used.
 *                            The system name shall have the ':' postfix.
 * @param dyn_string &exceptionInfo: exception returned by the function in case of error
 *                                    The exception is cleared at the beginning of the function
 * @return true if the manager is running. False if it is not running or in case of
 *          error (system name does not exist or DP does not exist)
 **/
bool unManager_isUnicosScriptsRunning( string sSystemName, dyn_string &dsExceptionInfo){
  dynClear(dsExceptionInfo);
  string sDpConnection = _unManager_getConnectionCtrlDp( sSystemName, dsExceptionInfo);
  if( dynlen(dsExceptionInfo)){
    return false;
  }

  dyn_int diList;
  int iRes = dpGet(sDpConnection, diList);
  return unManager_isUnicosScriptsInListOfRunningCtrl( diList);
}

/**
 * Given a list of running manager numbers,
 * check if the number associated to unDistributedControl.ctl
 * is present in this list.
 * The manager list is normally provided by the WinCC OA internal DPE
 * and this function is used as a callback on a dpConnnect on _Connections.Ctrl.ManNums
 * 
 * @param const dyn_int &diListCtrlManagerNumber: list of manager number
 *                                                the list should be the content of _Connections.Ctrl.ManNums
 * @return true if unDistributedControl.ctl manager number is in the list, false otherwise.
 **/
bool unManager_isUnDistributedControlInListOfRunningCtrl( const dyn_int &diListCtrlManagerNumber){
  return dynContains( diListCtrlManagerNumber, unManager_getUnDistributedControlManagerNumber()) > 0;
}

/**
 * Check if the manager number associated to unDistributedControl is 
 * in the list of running manager of the system given as parameter. 
 * 
 * @param string sSystemName: Optional name of the system to be checked.
 *                            If no name is given, the local system name is used.
 *                            The system name shall have the ':' postfix.
 * @param dyn_string &exceptionInfo: exception returned by the function in case of error
 *                                    The exception is cleared at the beginning of the function
 * @return true if the manager is running. False if it is not running or in case of
 *          error (system name does not exist or DP does not exist)
 **/
bool unManager_isUnDistributedControlRunning( string sSystemName, dyn_string &dsExceptionInfo){
  dynClear(dsExceptionInfo);
  string sDpConnection = _unManager_getConnectionCtrlDp( sSystemName, dsExceptionInfo);
  if( dynlen(dsExceptionInfo)){
    return false;
  }

  dyn_int diList;
  int iRes = dpGet(sDpConnection, diList);
  return unManager_isUnDistributedControlInListOfRunningCtrl( diList);
}

//-------------------------------------------------------------------------------------------------
private string _unManager_getConnectionCtrlDp( string sSystemName, dyn_string &dsExceptionInfo){
  if ( sSystemName==""){
    sSystemName = getSystemName();
  }

  string sDpConnection = sSystemName+"_Connections.Ctrl.ManNums";
  if (dpExists(sDpConnection)==false){
    fwException_raise(dsExceptionInfo, "ERROR", "["+__FUNCTION__ + "] DP does not exists: "+sDpConnection+" - Unable to check status of manager", "");
    return "";
  }

  return sDpConnection;
}